cwlVersion: v1.2
class: CommandLineTool
baseCommand: [filter_skymodel.py]
label: Filter a sky model
doc: |
  This tool uses PyBDSF to filter artifacts from the sky model and
  to attenuate the source flux densities to make an apparent-sky
  sky model.

requirements:
  - class: InlineJavascriptRequirement
  - class: InitialWorkDirRequirement
    listing:
      - entry: $(inputs.flat_noise_image)
        writable: true
      - entry: $(inputs.true_sky_image)
        writable: true

inputs:
  - id: flat_noise_image
    label: Detection image
    doc: |
      The filename of the input FITS image used for source detection.
    type: File
    inputBinding:
      position: 1
  - id: true_sky_image
    label: Input image
    doc: |
      The filename of the input primary-beam-corrected FITS image.
    type: File
    inputBinding:
      position: 2
  - id: true_sky_skymodel
    label: PB-corrected model
    doc: |
      The filename of the input primary-beam-corrected sky model.
    type: File
    inputBinding:
      position: 3
  - id: output_root
    label: Output root name
    doc: |
      The root of the filenames of the output filtered sky models.
    type: string
    inputBinding:
      position: 4
  - id: vertices_file
    label: Filename of vertices file
    doc: |
      The filename of the file containing sector vertices.
    type: File
    inputBinding:
      position: 5
  - id: beamMS
    label: Filenames of MS files for beam
    doc: |
      The filenames of the MS files to use for beam calculations. These MS files
      should have the original phase center of the observation.
    type: Directory[]
    inputBinding:
      position: 6
      itemSeparator: ","
  - id: bright_true_sky_skymodel
    label: Bright-source PB-corrected model
    doc: |
      The filename of the input bright-source primary-beam-corrected sky model. Should
      not be given if peeling of bright sources was not done.
    type: File?
    inputBinding:
      prefix: --bright_true_sky_skymodel=
      separate: false
  - id: threshisl
    label: Island threshold
    doc: |
      The PyBDSF island threshold.
    type: float
    inputBinding:
      prefix: --threshisl=
      separate: false
  - id: threshpix
    label: Pixel threshold
    doc: |
      The PyBDSF pixel threshold.
    type: float
    inputBinding:
      prefix: --threshpix=
      separate: false

outputs:
  - id: filtered_skymodel_true_sky
    label: Processed true-sky sky model
    doc: |
      The filename of the filtered true-sky sky model.
    type: File
    outputBinding:
      glob: '$(inputs.output_root).true_sky.txt'
  - id: filtered_skymodel_apparent_sky
    label: Processed apparent-sky sky model
    doc: |
      The filename of the filtered apparent-sky sky model.
    type: File
    outputBinding:
      glob: '$(inputs.output_root).apparent_sky.txt'
  - id: diagnostics
    label: Image diagnostics
    doc: |
      The image diagnostics, including number of sources.
    type: File
    outputBinding:
      glob: '$(inputs.output_root).image_diagnostics.json'
  - id: flat_noise_rms_image
    label: Flat-noise RMS image
    doc: |
      The flat-niose RMS image generated by PyBDSF.
    type: File
    outputBinding:
      glob: '$(inputs.output_root).flat_noise_rms.fits'
  - id: true_sky_rms_image
    label: True-sky RMS image
    doc: |
      The true-sky RMS image generated by PyBDSF.
    type: File
    outputBinding:
      glob: '$(inputs.output_root).true_sky_rms.fits'
  - id: source_catalog
    label: Source FITS catalog
    doc: |
      The source catalog generated by PyBDSF.
    type: File
    outputBinding:
      glob: '$(inputs.output_root).source_catalog.fits'


hints:
  - class: DockerRequirement
    dockerPull: 'astronrd/rapthor'
